auth_script

Attribute Macro auth_script 

#[auth_script]
Marks a component method as the authentication procedure entrypoint (#[auth_script]).

The method must be declared within a trait annotated with #[component]. Authentication components must annotate exactly one method with #[auth_script]. At most one method in a crate may be annotated with #[auth_script].